-- FILE        : RCC.vhd
-- DESCRIPTION : Receive, Check and Compare
-- DESIGNER    : Csaba Hajdu based on the schematic by Christos Zamantzas
-- VERSION     : 5
-- DATE        : 2010/03/29

--            ver. 1: RCC.bdf release 2010/01/11 converted to VHDL.
-- Changes in ver. 2: Cleanup of VHDL code
--                    Changes to accomodate combined ERR + CheckStatuses blocks
--                    Read enable signal chain moved into ERR.vhd
--                    ERRA and ERRB driven from ERR block instead of directly from the respective CRC_Check
-- Changes in ver. 3: counterSOF and counterSOF_EN merged and now inferred instead of being instantiated
--                      New LKdown condition: no Ready 32 clock cycles after SOF (frame transmission was interrupted)
-- Changes in ver. 4: LKdown now only gives pulses instead of continuous signal
--                      New counter to ensure presence of the pulses when both links are down
-- Changes in ver. 5: Muxes for the generation of CheckedSignal now inferred instead of being instantiated


LIBRARY ieee;
USE ieee.std_logic_1164.all; 
USE ieee.std_logic_unsigned.all; 

LIBRARY work;

ENTITY RCC IS 
    PORT
    (
        clk              :  IN  STD_LOGIC;
        clk_TLK_ChA      :  IN  STD_LOGIC;
        clk_TLK_ChB      :  IN  STD_LOGIC;
        ignoreCFC        :  IN  STD_LOGIC;
        reset            :  IN  STD_LOGIC;
        Status_ChA       :  IN  STD_LOGIC_VECTOR(1 DOWNTO 0);
        Data_ChA         :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
        Status_ChB       :  IN  STD_LOGIC_VECTOR(1 DOWNTO 0);
        Data_ChB         :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
        
        CheckedSignal    :  OUT  STD_LOGIC_VECTOR(79 DOWNTO 0);
        ReEn_Part_A      :  OUT  STD_LOGIC;
        ReEn_Part_B      :  OUT  STD_LOGIC;
        Dump             :  OUT  STD_LOGIC;
        ERRA             :  OUT  STD_LOGIC;
        ERRB             :  OUT  STD_LOGIC;
        ERRC             :  OUT  STD_LOGIC;
        ERRD             :  OUT  STD_LOGIC;
        ERRF             :  OUT  STD_LOGIC;
        StatusA          :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
        StatusB          :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
        SOF_ChA          :  OUT  STD_LOGIC;
        SOF_ChB          :  OUT  STD_LOGIC;
        CID              :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
        FID              :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
        DAC_1_2x         :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
        DAC_3_4x         :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
        DAC_5_6x         :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
        DAC_7_8x         :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
        D16_ChA          :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
        D16_ChB          :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
        Dvalid_ChA       :  OUT  STD_LOGIC;
        Dvalid_ChB       :  OUT  STD_LOGIC;
        ERRTLK_ChA       :  OUT  STD_LOGIC;
        ERRTLK_ChB       :  OUT  STD_LOGIC;
        LKdown_ChA       :  OUT  STD_LOGIC;
        LKdown_ChB       :  OUT  STD_LOGIC
    );
END RCC;


ARCHITECTURE bdf_type OF RCC IS 

COMPONENT err
    PORT(ERRA_in      :  IN  STD_LOGIC;
         ERRB_in      :  IN  STD_LOGIC;
         ERRC_in      :  IN  STD_LOGIC;
         LK_A         :  IN  STD_LOGIC;
         LK_B         :  IN  STD_LOGIC;
         Ready_ChA    :  IN  STD_LOGIC;
         Ready_ChB    :  IN  STD_LOGIC;
         procEN_ChA   :  IN  STD_LOGIC;
         procEN_ChB   :  IN  STD_LOGIC;
         CID_ChA      :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
         CID_ChB      :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
         CRC_ChA      :  IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
         CRC_ChB      :  IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
         FID_ChA      :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
         FID_ChB      :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
         StatusAx     :  IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
         StatusBx     :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
         ignoreCFC    :  IN  STD_LOGIC;
         clk          :  IN  STD_LOGIC;
         reset        :  IN  STD_LOGIC;
    
         ValidSignal  :  OUT  STD_LOGIC;
         ReEn_Part_A  :  OUT  STD_LOGIC;
         ReEn_Part_B  :  OUT  STD_LOGIC;
         rdErrors_out :  OUT  STD_LOGIC;
         rdAddr_out   :  OUT  STD_LOGIC;
         rdpart2x_out :  OUT  STD_LOGIC;
         ERRVoltage   :  OUT  STD_LOGIC;
         ERRCFC       :  OUT  STD_LOGIC;
         ERRA         :  OUT  STD_LOGIC;
         ERRB         :  OUT  STD_LOGIC;
         ERRC         :  OUT  STD_LOGIC;
         ERRD         :  OUT  STD_LOGIC;
         ERRF         :  OUT  STD_LOGIC;
         Dump         :  OUT  STD_LOGIC    
    );
END COMPONENT;

COMPONENT outputvalidstatuses
    PORT(validPacket  : IN STD_LOGIC;
         enableRead   : IN STD_LOGIC;
         clk          : IN STD_LOGIC;
         CID_ChA      : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         CID_ChB      : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_1_2_ChA  : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_1_2_ChB  : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_3_4_ChA  : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_3_4_ChB  : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_5_6_ChA  : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_5_6_ChB  : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_7_8_ChA  : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_7_8_ChB  : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         FID_ChA      : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         FID_ChB      : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         StatusA_ChA  : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         StatusA_ChB  : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         StatusB_ChA  : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         StatusB_ChB  : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         
         CID          : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_1_2x     : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_3_4x     : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_5_6x     : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_7_8x     : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
         FID          : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
         StatusA      : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
         StatusB      : OUT STD_LOGIC_VECTOR(15 DOWNTO 0)
    );
END COMPONENT;

COMPONENT rxbuffera
    PORT(wren         : IN STD_LOGIC;
         wrclock      : IN STD_LOGIC;
         rdclock      : IN STD_LOGIC;
         data         : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         rdaddress    : IN STD_LOGIC_VECTOR(0 TO 0);
         wraddress    : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
         
         q            : OUT STD_LOGIC_VECTOR(127 DOWNTO 0)
    );
END COMPONENT;

COMPONENT rxbufferb
    PORT(wren         : IN STD_LOGIC;
         wrclock      : IN STD_LOGIC;
         rdclock      : IN STD_LOGIC;
         data         : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         rdaddress    : IN STD_LOGIC_VECTOR(0 TO 0);
         wraddress    : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
         
         q            : OUT STD_LOGIC_VECTOR(127 DOWNTO 0)
    );
END COMPONENT;

COMPONENT crc_check
    PORT(CLK          : IN STD_LOGIC;
         RESET        : IN STD_LOGIC;
         D            : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
         status       : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
         
         DValid       : OUT STD_LOGIC;
         CRCready     : OUT STD_LOGIC;
         ERR          : OUT STD_LOGIC;
         ERR_TLK      : OUT STD_LOGIC;
         SOF          : OUT STD_LOGIC;
         calcCRC      : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
         CFCstatusA   : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
         CFCstatusB   : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
         CID          : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
         count        : OUT STD_LOGIC_VECTOR(4 DOWNTO 0);
         CRCout       : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
         D16out       : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_1_2x     : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_3_4x     : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_5_6x     : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
         DAC_7_8x     : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
         FID          : OUT STD_LOGIC_VECTOR(15 DOWNTO 0)
    );
END COMPONENT;

  SIGNAL    CID_ChA            :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    CID_ChB            :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    crc_ChA            :  STD_LOGIC_VECTOR(31 DOWNTO 0);
  SIGNAL    crc_ChB            :  STD_LOGIC_VECTOR(31 DOWNTO 0);
  SIGNAL    D16_ChA_int        :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    D16_ChB_int        :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    DAC_1_2_ChA        :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    DAC_1_2_ChB        :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    DAC_3_4_ChA        :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    DAC_3_4_ChB        :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    DAC_5_6_ChA        :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    DAC_5_6_ChB        :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    DAC_7_8_ChA        :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    DAC_7_8_ChB        :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    Dvalid_ChA_int     :  STD_LOGIC;
  SIGNAL    Dvalid_ChB_int     :  STD_LOGIC;
  SIGNAL    enRead             :  STD_LOGIC;
  SIGNAL    ERRA_int           :  STD_LOGIC;
  SIGNAL    ERRA_int_S         :  STD_LOGIC;
  SIGNAL    ERRB_int           :  STD_LOGIC;
  SIGNAL    ERRB_int_S         :  STD_LOGIC;
  SIGNAL    ERRCFC             :  STD_LOGIC;
  SIGNAL    ERRTLK_ChA_int     :  STD_LOGIC;
  SIGNAL    ERRTLK_ChA_int_S   :  STD_LOGIC;
  SIGNAL    ERRTLK_ChB_int     :  STD_LOGIC;
  SIGNAL    ERRTLK_ChB_int_S   :  STD_LOGIC;
  SIGNAL    ERRVoltage         :  STD_LOGIC;
  SIGNAL    FID_ChA            :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    FID_ChB            :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    LKdown_A           :  STD_LOGIC;
  SIGNAL    LKdown_B           :  STD_LOGIC;
  SIGNAL    LKdown_EN          :  STD_LOGIC;
  SIGNAL    Ready_ChA          :  STD_LOGIC;
  SIGNAL    Ready_ChA_int      :  STD_LOGIC := '0';
  SIGNAL    Ready_ChB          :  STD_LOGIC;
  SIGNAL    Ready_ChB_int      :  STD_LOGIC := '0';
  SIGNAL    procEN_ChA         :  STD_LOGIC;
  SIGNAL    procEN_ChB         :  STD_LOGIC;
  SIGNAL    resetN             :  STD_LOGIC;
  SIGNAL    SOF_ChA_int        :  STD_LOGIC;
  SIGNAL    SOF_ChB_int        :  STD_LOGIC;
  SIGNAL    StatusA_ChA        :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    StatusA_ChB        :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    StatusAx           :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    StatusB_ChA        :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    StatusB_ChB        :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    StatusBx           :  STD_LOGIC_VECTOR(15 DOWNTO 0);
  SIGNAL    validPacket        :  STD_LOGIC;
  SIGNAL    wrad_ChA           :  STD_LOGIC_VECTOR(4 DOWNTO 0);
  SIGNAL    wrad_ChB           :  STD_LOGIC_VECTOR(4 DOWNTO 0);
  SIGNAL    wren_ChA           :  STD_LOGIC;
  SIGNAL    wren_ChB           :  STD_LOGIC;
  SIGNAL    RxBufferA_out      :  STD_LOGIC_VECTOR(127 DOWNTO 0);
  SIGNAL    RxBufferB_out      :  STD_LOGIC_VECTOR(127 DOWNTO 0);
  SIGNAL    wrad_ChA4_n        :  STD_LOGIC;
  SIGNAL    wrad_ChB4_n        :  STD_LOGIC;
  SIGNAL    ERRA_int_D         :  STD_LOGIC;
  SIGNAL    ERRB_int_D         :  STD_LOGIC;
  SIGNAL    Ready_ChA_D        :  STD_LOGIC := '0';
  SIGNAL    Ready_ChB_D        :  STD_LOGIC := '0';
  SIGNAL    ReEn_Part_A_int    :  STD_LOGIC;
  SIGNAL    ReEn_Part_B_int    :  STD_LOGIC;
  SIGNAL    ReEn_Part_B_D      :  STD_LOGIC;
  SIGNAL    ERRTLK_ChA_int_D   :  STD_LOGIC;
  SIGNAL    ERRTLK_ChB_int_D   :  STD_LOGIC;
  SIGNAL    rdErrors           :  STD_LOGIC;
  SIGNAL    rdAddr             :  STD_LOGIC;
  SIGNAL    rdpart2x           :  STD_LOGIC;


BEGIN 


-- CRC Checker modules         
crc_check_inst1 : crc_check
PORT MAP(CLK        => clk_TLK_ChA,
         RESET      => reset,
         D          => Data_ChA,
         status     => Status_ChA,
         DValid     => Dvalid_ChA_int,
         CRCready   => Ready_ChA,
         ERR        => ERRA_int,
         ERR_TLK    => ERRTLK_ChA_int,
         SOF        => SOF_ChA_int,
         calcCRC    => crc_ChA,
         CFCstatusA => StatusA_ChA,
         CFCstatusB => StatusB_ChA,
         CID        => CID_ChA,
         count      => wrad_ChA,
         D16out     => D16_ChA_int,
         DAC_1_2x   => DAC_1_2_ChA,
         DAC_3_4x   => DAC_3_4_ChA,
         DAC_5_6x   => DAC_5_6_ChA,
         DAC_7_8x   => DAC_7_8_ChA,
         FID        => FID_ChA);

crc_check_inst2 : crc_check
PORT MAP(CLK        => clk_TLK_ChB,
         RESET      => reset,
         D          => Data_ChB,
         status     => Status_ChB,
         DValid     => Dvalid_ChB_int,
         CRCready   => Ready_ChB,
         ERR        => ERRB_int,
         ERR_TLK    => ERRTLK_ChB_int,
         SOF        => SOF_ChB_int,
         calcCRC    => crc_ChB,
         CFCstatusA => StatusA_ChB,
         CFCstatusB => StatusB_ChB,
         CID        => CID_ChB,
         count      => wrad_ChB,
         D16out     => D16_ChB_int,
         DAC_1_2x   => DAC_1_2_ChB,
         DAC_3_4x   => DAC_3_4_ChB,
         DAC_5_6x   => DAC_5_6_ChB,
         DAC_7_8x   => DAC_7_8_ChB,
         FID        => FID_ChB);         

  Dvalid_ChA <= Dvalid_ChA_int;
  Dvalid_ChB <= Dvalid_ChB_int;
  SOF_ChA <= SOF_ChA_int;
  SOF_ChB <= SOF_ChB_int;
  D16_ChA <= D16_ChA_int;
  D16_ChB <= D16_ChB_int;
 
 
 
-- Synchronizers
PROCESS(clk)
  BEGIN
    IF (RISING_EDGE(clk)) THEN
      ERRA_int_D <= ERRA_int;
      ERRA_int_S <= ERRA_int_D;

      ERRB_int_D <= ERRB_int;      
      ERRB_int_S <= ERRB_int_D;

      Ready_ChA_D <= Ready_ChA;
      Ready_ChA_int <= Ready_ChA_D;

      Ready_ChB_D <= Ready_ChB;
      Ready_ChB_int <= Ready_ChB_D;

      ERRTLK_ChA_int_D <= ERRTLK_ChA_int;
      ERRTLK_ChA_int_S <= ERRTLK_ChA_int_D;
    
      ERRTLK_ChB_int_D <= ERRTLK_ChB_int;
      ERRTLK_ChB_int_S <= ERRTLK_ChB_int_D;
    END IF;
  END PROCESS;         

  ERRTLK_ChA <= ERRTLK_ChA_int_S;
  ERRTLK_ChB <= ERRTLK_ChB_int_S;



-- Error signal and dump generation
err_inst : err
PORT MAP(ERRA_in      => ERRA_int_S,
         ERRB_in      => ERRB_int_S,
         ERRC_in      => '0',
         LK_A         => LKdown_A,
         LK_B         => LKdown_B,
         Ready_ChA    => Ready_ChA_int,
         Ready_ChB    => Ready_ChB_int,
         procEN_ChA   => procEN_ChA,
         procEN_ChB   => procEN_ChB,
         CID_ChA      => CID_ChA,
         CID_ChB      => CID_ChB,
         CRC_ChA      => crc_ChA,
         CRC_ChB      => crc_ChB,
         FID_ChA      => FID_ChA,
         FID_ChB      => FID_ChB,
         StatusAx     => StatusAx(3 DOWNTO 0),
         StatusBx     => StatusBx,
         ignoreCFC    => ignoreCFC,
         clk          => clk,
         reset        => reset,
    
         ValidSignal  => validPacket,
         ReEn_Part_A  => ReEn_Part_A_int,
         ReEn_Part_B  => ReEn_Part_B_int,
         rdErrors_out => rdErrors,
         rdAddr_out   => rdAddr,
         rdpart2x_out => rdpart2x,
         ERRVoltage   => ERRVoltage,
         ERRCFC       => ERRCFC,
         ERRA         => ERRA,
         ERRB         => ERRB,
         ERRC         => ERRC,
         ERRD         => ERRD,
         ERRF         => ERRF,
         Dump         => Dump);

         
-- Receive buffers and muxes for CheckedSignal transmission  
  wrad_ChA4_n <= NOT (wrad_ChA(4));
  wren_ChA    <= Dvalid_ChA_int AND wrad_ChA4_n;

  wrad_ChB4_n <= NOT (wrad_ChB(4));
  wren_ChB    <= Dvalid_ChB_int AND wrad_ChB4_n;
  
rxbuffera_inst : rxbuffera
PORT MAP(wren         => wren_ChA,
         wrclock      => clk_TLK_ChA,
         rdclock      => clk,
         data         => D16_ChA_int,
         rdaddress(0) => rdAddr,
         wraddress    => wrad_ChA(3 DOWNTO 0),
         q            => RxBufferA_out);

rxbufferb_inst : rxbufferb
PORT MAP(wren         => wren_ChB,
         wrclock      => clk_TLK_ChB,
         rdclock      => clk,
         data         => D16_ChB_int,
         rdaddress(0) => rdAddr,
         wraddress    => wrad_ChB(3 DOWNTO 0),
         q            => RxBufferB_out);

  CheckedSignal <= RxBufferA_out(127 DOWNTO 48)      WHEN (validPacket = '0') AND (rdpart2x = '0')
              ELSE RxBufferA_out (79 DOWNTO  0)      WHEN (validPacket = '0') AND (rdpart2x = '1')
              ELSE RxBufferB_out(127 DOWNTO 48)      WHEN (validPacket = '1') AND (rdpart2x = '0')
              ELSE RxBufferB_out (79 DOWNTO  0);  -- WHEN (validPacket = '1') AND (rdpart2x = '1')

  
  resetN <= NOT (reset);

  
-- Start Of Frame counters: Processing Enable and Link Down signals
-- procEN asserted 32 clock cycles after SOF -> if one of the links has just gone down, this will have been detected by then
PROCESS (clk, SOF_ChA_int)
  VARIABLE Count       : STD_LOGIC_VECTOR(10 DOWNTO 0);
  VARIABLE procEN      : STD_LOGIC;
  
  BEGIN
    IF (reset = '1') OR (SOF_ChA_int = '1') THEN
      Count    := (OTHERS => '0');
      procEN   := '0';
      LKdown_A <= '0';
    ELSIF RISING_EDGE(clk) THEN
      -- Link Down: no SOF for too long
      IF (Count < 1632) THEN
        LKdown_A <= '0';
        Count := Count + 1;
      ELSE
        LKdown_A <= '1';
      END IF;
      
      -- Processing Enable: 32 clock cycles elapsed since SOF (wait for eventual LKdown on other channel)
      IF (Count < 32) THEN
        procEN := '0';
      ELSE
        procEN := '1';
      END IF;

      procEN_ChA <= procEN;      

      -- Link Down: no Ready signal 32 clock cycles after SOF
      IF (procEN = '1') AND (Ready_ChA = '0') THEN
        LKdown_A <= '1';
      END IF;
    END IF;  
  END PROCESS;
  
PROCESS (clk, SOF_ChB_int)
  VARIABLE Count       : STD_LOGIC_VECTOR(10 DOWNTO 0);
  VARIABLE procEN      : STD_LOGIC;
  
  BEGIN
    IF (reset = '1') OR (SOF_ChB_int = '1') THEN
      Count    := (OTHERS => '0');
      procEN   := '0';
      LKdown_B <= '0';
    ELSIF RISING_EDGE(clk) THEN
      -- Link Down: no SOF for too long
      IF (Count < 1632) THEN
        LKdown_B <= '0';
        Count := Count + 1;
      ELSE
        LKdown_B <= '1';
      END IF;
      
      -- Processing Enable: 32 clock cycles elapsed since SOF (wait for eventual LKdown on other channel)
      IF (Count < 32) THEN
        procEN := '0';
      ELSE
        procEN := '1';
      END IF;

      procEN_ChB <= procEN;      

      -- Link Down: no Ready signal 32 clock cycles after SOF
      IF (procEN = '1') AND (Ready_ChB = '0') THEN
        LKdown_B <= '1';
      END IF;
    END IF;  
  END PROCESS;

-- Create single pulses on LKdown_ChA and LKdown_ChB outputs for correct Lost Frames count
PROCESS (clk)
  BEGIN
    IF RISING_EDGE(clk) THEN
      LKdown_ChA <= '0';
      LKdown_ChB <= '0';

      IF ( (ReEn_Part_B_D = '0') AND (ReEn_Part_B_int = '1') ) OR                   -- Release LKdown pulses simultaneously with error flags
         ( (LKdown_A = '1') AND (LKdown_B = '1') AND (LKdown_EN = '1') ) THEN       -- OR Release pulses at LKdown_EN if both links are down (ReEn signals don't get asserted)
        LKdown_ChA <= LKdown_A;
        LKdown_ChB <= LKdown_B;
      END IF;
          
      ReEn_Part_B_D <= ReEn_Part_B_int;
    END IF;
  END PROCESS;

  ReEn_Part_A <= ReEn_Part_A_int;
  ReEn_Part_B <= ReEn_Part_B_int;

-- Free-running mod 1600 counter reset 32 clock cycles after a SOF on either channel allowing to keep track of lost frames when both links are down
PROCESS (clk)
  VARIABLE  Cnt       :  INTEGER RANGE 0 to 1599;
  VARIABLE  procEN    :  STD_LOGIC;
  VARIABLE  procEN_D  :  STD_LOGIC;
  
  BEGIN
    IF (procEN = '1') AND (procEN_D = '0') THEN
      Cnt := 0;
      LKdown_EN <= '0';
    ELSIF RISING_EDGE(clk) THEN
      IF Cnt < 1599 THEN
        Cnt := Cnt + 1;
        LKdown_EN <= '0'; 
      ELSE
        Cnt := 0;
        LKdown_EN <= '1'; 
      END IF;
    END IF;  

    IF RISING_EDGE(clk) THEN
      procEN_D := procEN;
      procEN   := procEN_ChA AND procEN_ChB;
    END IF;
  END PROCESS;

  
-- Status output
outputvalidstatuses_inst : outputvalidstatuses
PORT MAP(validPacket => validPacket,
         enableRead  => rdErrors,
         clk         => clk,
         CID_ChA     => CID_ChA,
         CID_ChB     => CID_ChB,
         DAC_1_2_ChA => DAC_1_2_ChA,
         DAC_1_2_ChB => DAC_1_2_ChB,
         DAC_3_4_ChA => DAC_3_4_ChA,
         DAC_3_4_ChB => DAC_3_4_ChB,
         DAC_5_6_ChA => DAC_5_6_ChA,
         DAC_5_6_ChB => DAC_5_6_ChB,
         DAC_7_8_ChA => DAC_7_8_ChA,
         DAC_7_8_ChB => DAC_7_8_ChB,
         FID_ChA     => FID_ChA,
         FID_ChB     => FID_ChB,
         StatusA_ChA => StatusA_ChA,
         StatusA_ChB => StatusA_ChB,
         StatusB_ChA => StatusB_ChA,
         StatusB_ChB => StatusB_ChB,
         CID         => CID,
         DAC_1_2x    => DAC_1_2x,
         DAC_3_4x    => DAC_3_4x,
         DAC_5_6x    => DAC_5_6x,
         DAC_7_8x    => DAC_7_8x,
         FID         => FID,
         StatusA     => StatusAx,
         StatusB     => StatusBx);

  StatusA <= StatusAx;
  StatusB <= StatusBx;

END bdf_type;